;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 49A2			;18850

@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@Laddr=09D8			;Line address subroutine
@DFile=400C			;Address of display file
@Spare1=4021			;16417 Spare 1, 1 Byte
@Seed=4032			;16 bit parameter passing via RAND
@LastK=4025			;Keyboard debounce value
@LoopFlag=4083			;16515, Player stopped flag
@WorkArea=BF9A			;490150, work area

;================================
; Jump Table
;================================
	JP KeyLoop		;18850, Key loop and sorter
	JR LoadScr		;18853, Load main form
	JR SaveScr		;18855, Save main form
	JR Stopped		;18857, Test "Stopped" flag
	JR Release		;18859, Test for keypress
	JR FormSet		;18861, Set form addresses
	JP PrntPos		;18863,	Calculate Print Position
	JP FileDrop		;18866, File dropdown menu
	JP VidDrop		;18869, Videos dropdown menu
	JP WallDrop		;18872, Walls dropdown menu
	JP HelpDrop		;18875, Help dropdown menu
	JP TrackNum		;18878, Convert track number to w on b
	JP PrntInfo		;18881, Print PT3 information
	00 00 00		;18883, Extra
@MainForm
	00 00
	00 00			;Extras
@AltDFile			;Warning, adjust @AltDFile in Videos.asm if this location moves
	00 00
;================================
; Forms set
;================================
@FormSet
	LD HL,0003		;Locate line 3
	CALL LAddr		;Get line address
	LD BC,0006
	ADD HL,BC
	LD (MainForm),HL	;Set main form address
	LD BC,0318
	ADD HL,BC
	LD (AltDFile),HL	;Set AltDFile address
;================================
; Load screen
;================================
@LoadScr
	LD HL,(MainForm)	;Set source
	LD DE,(DFile)		;Set destination
	JR +7
;================================
; Save screen
;================================
@SaveScr
	LD HL,(DFile)		;Set source
	LD DE,(MainForm)	;Set destination
	LD BC,0319		;BC=byte count
	LDIR			;Copy source to desination
	RET			;Return 0
;================================
; Wait for key release
;================================
@Release
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ Release		;Yes, loop back
	RET
;================================
; Test "Stopped" value and keypress
; Display wallpaper until tune stopped
; or keypressed.
;================================
@Stopped
	CALL Release		;Wait for no key press
	LD HL,LoopFlag		;Loopflag addr
	BIT 7,(HL)		;Test stopped bit.
	JR Z +6			;If 0 move on, still playing
	RES 7,(HL)		;Tune ended, reset stopped flag
	BIT 0,(HL)		;Test loop flag
	JR NZ +9		;If NZ don't loop
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ +2		;Yes, return
	JR Stopped+3		;No, quit if keypressed
	CALL Release		;Test for key press
	CALL LoadScr		;Restore main screen
	LD C,5A			;BC=90
	RET			;GOTO 90
;================================
; Print dropdown menu
;================================
@DropDown
	CALL SaveScr		;Save main screen
	LD HL,(Seed)		;Get line number to locate
	CALL Release		;Wait for key release
	CALL LAddr		;Get line address
	LD BC,0005		;Skip REM header
	ADD HL,BC		;Start of data
	LD B,(HL)		;Get Ycoord
	INC HL
	LD C,(HL)		;Get Xcoord
	INC HL			;Start of print data
	PUSH HL			;PUSH datapointer
	CALL PrntPos		;Get print position of the NW corner of dropdown
	POP BC			;POP Datapointer into BC
@DropY
	LD (Drop+1),HL		;Save start of row addr
@DropLoop
	LD A,(BC)		;Get chr to print
	CP 76			;EO REM?
	RET Z			;Return printing done
	CP 0C			;EOL marker ()?
	JR Z Drop		;Jump if EOL
	LD (HL),A		;Print A
;	JR NZ +7
	JR +7
@Drop
	LD HL,0000		;Get start of row addr
	ADD HL,DE		;Next row addr
	INC BC			;Increase data pointer
	JR DropY		;Start of row addr
	INC HL			;Increase print position
	INC BC			;Increase data pointer
	JR DropLoop
;================================
; File menu dropdown
;================================
@FileDrop
	CALL DropDown		;Print file dropdown menu
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	LD HL,FileKeys		;Valid keys table
	LD BC,0014 		;Byte count
	CPIR 			;Locate key
	JR NZ +1		;Jump if no match
	INC B			;B=Hi byte of line number
	CALL Release		;Wait for key release
	LD C,(HL)		;C=Lo byte of line number
	RET			;Return line number to BASIC
;================================
@FileKeys			;Key code/lo byte of line number
	"O" 90			;400
	"P" 9A			;410
	"B" 40			;320
	"N" 4A			;330
	"L" 54			;340
	"S" 5E			;350
	"I" 68			;360
	"T" 72			;370
	"M" 7C			;380
	"C" 86			;390
	5A			;EOTable marker=GOTO 90
;================================
; Videos menu dropdown
;================================
@VidDrop
	CALL DropDown		;Print Video dropdown menu
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	LD HL,VidTable		;Valid keys table
	LD BC,0022 		;Byte count
	CPIR 			;Locate key
	JR NZ +2		;Jump if no match
	LD B,4D			;B=Hi byte of USR call
	CALL Release		;Wait for key release
	LD C,(HL)		;C=Lo byte of USR call
	RET			;Return USR call to BASIC
;================================
@VidTable			;Key code followed by lo byte of USR call
	"A" 00			;4D00
	"B" 03			;4D03
	"G" 06			;4D06
	"C" 2C			;4D2C
	"D" 2F			;4D2F
	"E" 0C			;4D0C
	"H" 09			;4D09
	"L" 0F			;4D0F
	"K" 12			;4D12
	"P" 15			;4D15
	"R" 18			;4D18
	"S" 29			;4D29
	"T" 1B			;4D1B
	"U" 1E			;4D1E
	"V" 26			;4D26
	"W" 21			;4D21
	"Z" 23			;4D23
	5A			;EOTable marker=GOTO 90
;================================
; Walls menu dropdown
;================================
@WallDrop
	LD HL,(Seed)		;REM line 1000
	CALL DropDown		;Print walls dropdown
	LD H,03			;Hi byte of REM line
	LD A,(Spare1)		;Get current set number
	ADD A,4B		;Adjust set number
	LD L,A			;Lo byte of REM line, HL=1000, 1001, 1002...
	CALL DropDown+9		;Print dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	LD BC,0000		;Initialize BC
	CP "A"			;Key<"A"?
	JR NC +4		;Jump if key>"9"
@DropRet
	CALL LoadScr		;Reload main screen
	RET
	CP "W"			;Key>="W"
	JR NC DropRet		;Jump if key>"V"
	LD B,24			;Calculate print position
	LD C,0F			;XCoord
	SUB B			;Row=Key code-36
	LD B,A			;YCoord
	CALL PrntPos		;Get print positon
	CALL Release		;Wait for key release
	LD A,(HL)		;Test for next wall selected, key "V"
	CP BB			;"gV"?
	INC HL
	INC HL			;Start of wall file name
	JR NZ WallName		;Jump if not key "V"
	LD DE,0004
	ADD HL,DE		;Point to set number
	LD A,(HL)		;Get set number
	LD (Spare1),A		;Set Spare1 to set number
	JR WallDrop+6		;Print next set dropdown
@WallName			;Return pointer to wall name
	LD B,H
	LD C,L			;BC=Wall name DFile address
	RET			;Return DFile address to BASIC
;================================
; Help menu dropdown
;================================
@HelpDrop
	CALL DropDown		;Print help dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CP "H"			;"H"ot keys
	JR NZ +10
	CALL Release		;Wait for key release
	CALL LoadScr		;Reload main screen
	LD BC,+630
	RET			;GOTO 730
	CP "K"			;"K"ey Guide
	JR NZ +2
	JR KeyGuide
	CP "A"			;"A"bout
	JR Z About
	LD BC,005A
	RET			;GOTO 90
;================================
; Key Guide
;================================
@KeyGuide
	LD HL,+615		;REM 615
	JR +3
;================================
; About
;================================
@About
	LD HL,+620		;REM 620
	LD (Seed),HL		;Set (SEED)
	CALL LoadScr		;Reload main screen
	CALL DropDown+3		;Print REM 715 or 720
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CALL Release		;Wait for key release
	CALL LoadScr		;Reload main screen
	LD C,5A
	RET			;GOTO 90
;================================
; Print Position
;  B=Ycoord
;  C=Xcoord
;  BC unchange
;================================
@PrntPos
	PUSH BC			;PUSH YXcoords
	LD HL,0000		;Set accumulator to 0
	LD DE,0021		;Bytes/row
	LD A,B
	AND A			;Row=0?
	JR Z +3			;Yes, move on
	ADD HL,DE		;HL=33*row value
	DJNZ -3
	ADD HL,BC		;HL=33*row value+column offset
	LD BC,(DFile)		;Get DFile
	INC BC			;Skip 1st N/L chr
	ADD HL,BC		;HL=print position
	POP BC			;POP YXcoords
	RET	
;================================
; Test for keypress.
; On exit...
;   A=chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress, D=255 if no key press
	INC D 			;0?
	RET Z
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	INC A			;Set Z flag
	LD A,(HL)
	RET			;Return chr code to BASIC
;================================
; Key loop
;================================
@KeyLoop
	CALL TrackName		;Track name display
	LD HL,LoopFlag		;Loopflag addr
	BIT 7,(HL)		;Test stopped bit.
	JR Z KeyTest		;If 0 move on, still playing
	RES 7,(HL)		;Tune ended, reset stopped flag
	BIT 0,(HL)		;Test loop flag
	JR Z KeyLoop		;If Z loop
	LD BC,0064		;If NZ start next tune, BC=100
	RET			;GOTO 100
@KeyTest
	CALL KeyScan		;Get key press
	JR Z KeyLoop		;No, loop until key pressed
	CP "F"			;"F"ile dropdown?
	JR NZ +4
	LD BC,+250
	RET			;GOTO 250
	CP "V"			;"V"ideo dropdown?
	JR NZ +4
	LD BC,+500
	RET			;GOTO 500
	CP "W"			;"W"alls dropdown?
	JR NZ +7
	CALL SaveScr		;Save screen
	LD BC,+800
	RET			;GOTO 800
	CP "H"			;"H"elp dropdown?
	JR NZ +4
	LD BC,+600
	RET			;GOTO 600
	CP "X"			;E"x"it key?
	JR NZ +4
	LD BC,+700
	RET			;GOTO 700
	LD HL,FileKeys		;Hot keys table
	LD BC,0012 		;Byte count
	CPIR 			;Locate key
	JR NZ  Keyloop		;Loop if none found
	CP "B"			;"B"ack?
	JR Z +2
	CP "N"			;"N"ext
	CALL NZ Release		;Wait for key release except for "B" & "N"
	LD B,01 		;Hi byte line number
	LD C,(HL) 		;Lo byte line number
	RET			;GOTO BC
;================================
; Track Name Display
;================================
@TrackName
	LD HL,(DFile)		;Get DFile
	LD DE,00D0		;Offset to track name
	ADD HL,DE		;HL=Track name location
	LD D,H			;DE=Track name location
	LD E,L
	LD B,0C			;Byte count
;NameLoop
	LD A,(HL)		;Get chr at HL
	ADD A,80		;Add 128
	LD (HL),A		;Set new chr
	INC HL			;Next chr
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR Z +9			;Jump to delay if no keypress
	EX DE,HL		;Set HL to track name location
	LD B,0C			;Byte count
	RES 7,(HL)		;Reset track name
	INC HL
	DJNZ -5
	RET
;Delay
	PUSH BC			;PUSH byte count
	LD BC,0A01		;Delay loop
	DEC BC 
	LD A,B 
	OR C 
	JR NZ -5
	POP BC			;POP byte count
	DJNZ -33		;Jump to NameLoop until B=0
	RET
;================================
; Track Number Display
;================================
@TrackNum
	LD BC,1718
	CALL PrntPos
	LD B,08
	SET 7,(HL)
	INC HL
	DJNZ -5
	LD C,00
	RET
;================================
;Print Header Info
;================================
@PrntInfo
	LD HL,801E		;Copy PT3 data, line 1
	LD DE,WorkArea
	LD BC,0020		;Byte count,32
	LDIR
	INC HL			;Copy PT3 data, line 2
	LD C,24			;Byte count
	LDIR
	LD HL,4CDC		;Erase this section
	LD C,1C			;Byte count, 27
	LDIR
	LD HL,WorkArea		;Get start of PT3 data
	DEC HL			;PT3 data-1
	LD B,63			;Byte count
@ASC01				;ASCII to ZX81 conversion
	PUSH BC			;PUSH counter
	INC HL
	LD A,(HL)
	CP 41			;Code<A?
	JR C Numeral		;Yes, move on
	CP 41			;Code<A?
	JR C Lower		;Yes, move on
	CP 5B			;Code>Z?
	JR NC Lower		;Yes, move on
	SUB 1B			;Must be an uppercase chr.
	JR Change		;Convert it to ZX81
@Lower
	CP 61			;Code<a?
	JR C Symbol		;Yes, move on
	CP 7B			;Code>z?
	JR NC Symbol		;Yes, move on
	SUB 3B			;Must be a lowercase chr.
	JR Change		;Convert it to ZX81
@Numeral
	CP 30			;Code<0?
	JR C Symbol		;Yes, move on
	CP 3A			;Code>9?
	JR NC Symbol		;Yes, move on
	SUB 14			;Must be a numeral
	JR Change		;Convert it to ZX81
@Symbol
	EX DE,HL		;Save char address
	LD BC,0046		;Table size
	LD HL,Symbolz
	CPIR			;Locate A in table
	LD A,(HL)		;Get the conversion value
	EX DE,HL		;Restore chr address
	JR Z Change		;Unsupported ASCII chr
	LD A,80			;Unsupported ASCII character
@Change
	SET 7,A
	LD (HL),A
	POP BC			;POP counter
	DJNZ ASC01
	LD HL,(DFile)		;Get DFile
	LD BC,0274		;Offset to print position
	ADD HL,BC		;Print position
	LD DE,WorkArea
	EX DE,HL		;Set pointers for LDIR
	LD A,03			;3 lines to print
	LD B,00
@NxtLine
	LD C,20			;32 Bytes/line
	LDIR			;Copy line to DFile
	INC DE			;Skip N/L
	DEC A			;Decrease counter
	JR NZ NxtLine
	RET
;================================
@Symbolz
	20 00			; Space
	22 0B			; "
	A3 0C			; 
	24 0D			; $
	3A 0E			; :
	3F 0F			; ?
	28 10			; (
	29 11			; )
	3E 12			; >
	3C 13			; <
	3D 14			; =
	2B 15			; +
	2D 16			; -
	2A 17			; *
	2F 18			; /
	3B 19			; ;
	2C 1A			; ,
	2E 1B			; .
;================================
;Replacement symbols
	09 00			; TAB with space		
	7E 16			; ~ with -
	60 0B			; ` with "
	21 1D			; ! with 1
	40 17			; @ with *
	23 15			; # with +
	25 86			; % with NW/SE double pixel
	5E 02			; ^ with NE pixel
	26 0C			; & with 
	5F 09			; _ with Gray Bar
	5B 10			; [ with (
	7B 10			; { with (
	5D 11			; ] with )
	7D 11			; } with )
	5C 18			; \ with /
	7C 05			; | with left black bar
	27 0B			; ' with "
